#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/slab.h>

struct mydata {
  int int_param;
  long long_param;
};

static int __init slab_allocator_kernel_module_init(void)
{
  int ret = 0;
  struct mydata *ptr1, *ptr2;
  struct kmem_cache *mycache;

  printk("slab_allocator_kernel_module_init()\n");
  mycache = kmem_cache_create("lkp-cache", sizeof(struct mydata), 0, 0, NULL);

  if (!mycache) {
    return -1;
  }

  ptr1 = kmem_cache_alloc(mycache, GFP_KERNEL);

  if (!ptr1) {
    ret = -ENOMEM;
    goto destroy_cache;
  }

  ptr2 = kmem_cache_alloc(mycache, GFP_KERNEL);

  if (!ptr2) {
    ret = -ENOMEM;
    goto freeptr1;
  }

  ptr1->int_param = 1;
  ptr1->long_param = 2;
  ptr2->int_param = 3;
  ptr2->long_param = 4;
  printk("ptr1 = {%d, %ld}, ptr2 = {%d, %ld}\n", ptr1->int_param,
         ptr1->long_param, ptr2->int_param, ptr2->long_param);
  kmem_cache_free(mycache, ptr2);

freeptr1:
  kmem_cache_free(mycache, ptr1);
destroy_cache:
  kmem_cache_destroy(mycache);
  return ret;
}

static void __exit slab_allocator_kernel_module_exit(void)
{
  printk("slab_allocator_kernel_module_exit()\n");
}

module_init(slab_allocator_kernel_module_init);
module_exit(slab_allocator_kernel_module_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Hiroyuki Chishiro");
MODULE_DESCRIPTION("Slab Allocator Kernel Module");
